#
# Regions
#

# todo add region consts


#
# Networking
#

QUAD_ZERO = '0.0.0.0/0'
VNET_CIDR_8 = '10.0.0.0/8'
VNET_CIDR_16 = '10.0.0.0/16'
VNET_CIDR_24 = '10.0.0.0/24'

SSH_PORT = 22
MONGODB_PORT = 27017
NTP_PORT = 123
SMTP_PORT_25 = 25
SMTP_PORT_587 = 587
HTTP_PORT = 80
HTTPS_PORT = 443
REDIS_PORT = 6379
MEMCACHED_PORT = 11211
POSTGRESQL_PORT = 5432

TCP_PROTOCOL = 6
UDP_PROTOCOL = 17
ICMP_PROTOCOL = 1
ALL_PROTOCOL = -1

#
# Azure virtual machine sizes
#

# todo add vm sizes

#
# Parameter types
#

STRING = 'String'
NUMBER = 'Number'
LIST_OF_NUMBERS = 'List<Number>'
COMMA_DELIMITED_LIST = 'CommaDelimitedList'
